﻿// Copyright (c) "Neo4j"
// Neo4j Sweden AB [https://neo4j.com]
// 
// Licensed under the Apache License, Version 2.0 (the "License").
// You may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using FluentAssertions;
using Xunit;

namespace Neo4j.Driver.IntegrationTests.Stress;

public sealed class BlockingWrongCommandTxFunc : BlockingCommand
{
    public BlockingWrongCommandTxFunc(IDriver driver)
        : base(driver, false)
    {
    }

    public override void Execute(StressTestContext context)
    {
        using var session = NewSession(AccessMode.Read, context);

        session.ExecuteRead(
            txc =>
            {
                var result = txc.Run("RETURN");
                var exc = Record.Exception(() => result.Consume());
                exc.Should()
                    .BeOfType<ClientException>()
                    .Which.Code.Should()
                    .Be("Neo.ClientError.Statement.SyntaxError");

                return result;
            });
    }
}
