﻿// Copyright (c) "Neo4j"
// Neo4j Sweden AB [https://neo4j.com]
// 
// Licensed under the Apache License, Version 2.0 (the "License").
// You may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;

namespace Neo4j.Driver.Internal.Util;

internal class ConcurrentHashSet<T> : IEnumerable<T>
{
    private readonly ConcurrentDictionary<T, bool> _items = new();

    public int Count => _items.Count;

    public IEnumerator<T> GetEnumerator()
    {
        return _items.Keys.GetEnumerator();
    }

    IEnumerator IEnumerable.GetEnumerator()
    {
        return GetEnumerator();
    }

    /// <summary>true if the item was added to the set successfully; false if the item already exists.</summary>
    /// <param name="item"></param>
    /// <returns></returns>
    public bool TryAdd(T item)
    {
        return _items.GetOrAdd(item, _ => true);
    }

    /// <summary>true if the item was removed from the set successfully; false if the item does not exists.</summary>
    /// <param name="item"></param>
    /// <returns></returns>
    public bool TryRemove(T item)
    {
        return _items.TryRemove(item, out _);
    }

    public override string ToString()
    {
        return _items.Keys.ToContentString();
    }
}
